//
// Copyright (C)  2006-2009 - CEA - Jean-Marc Martinez
// Copyright (C)  2009 - INRIA - Michael Baudin
//
// This file is released under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html
//
#include <iostream>
#include <sstream>
#include <fstream>
#include <vector>

#define _USE_MATH_DEFINES
#include "math.h"

using namespace std;

#include "nisp_util.h"
#include "nisp_math.h"
#include "nisp_smolyak.h"
#include "nisp_msg.h"
#include "nisp_qua.h"

// Methode du trapeze (bords retires n = 2 ** (l-1) + 1)
void TrapezoidalRule(double x[], double w[], int n) {
	int i;
	double delta = 1./ (n + 1.);
	for(i=0;i<n;i++) {
		x[i]= (i+1.)*delta; w[i]=1./n;
	}
}

// Methode de Fejer
void FejerRule(double x[], double w[], int n) {
	int i,j;
	double theta;
	for(i=0;i<n;i++) {
		theta=(i+1.)*M_PI/(n+1.);
		x[i]=cos(theta);
		for(w[i]=0,j=0;j<(n+1)/2;j++) {
			w[i]+=sin((2.*j+1.)*theta)/(2.*j+1.);
		}
		w[i]=w[i]*sin(theta)*4./(n+1.);
	}
	for(i=0;i<n;i++) {
		x[i]=(x[i]+1.)/2.; 
		w[i]=w[i]/2.;
	} // entre [0,1]
}

// Methode de Clenshaw Curtis
void ClenshawCurtisRule(double x[], double w[], int n) {
	int i,j;
	double theta;
	if(n==1) {x[0]=0.; w[0]=2.;}
	else { // ici n est > 2
		for(i=0;i<n;i++) {
			x[i]=-cos(M_PI * i / (n-1));
		}
		w[0]=w[n-1]=1./(n*(n-2.));
		for(i=1;i<n-1;i++) {
			theta=M_PI * i / (n-1);
			for(w[i]=0.,j=1;j<=(n-1)/2-1;j++) {
				w[i]+=2./(1-4*j*j) * cos(2.*theta*j);
			}
			w[i]+=1./(1-4*j*j) * cos(theta*(n-1));
			w[i]=(2./(n-1))*(1.+w[i]);
		}
	}
	for(i=0;i<n;i++) {
		x[i]=(x[i]+1.)/2.; 
		w[i]=w[i]/2.;
	} // entre [0,1]
}


// incrementation du multi indice (k(0),k(1),...,k(nx-1))
// min(i)<= k(i) <= max(i)
// Remarque : avant l'appel l'initialisation doit avoir ete effectuee
// Valeur initiale du multi indice (min(0),min(1),...,min(nx-1))
// retour 1 si l'incrementation a ete realisee 
// retour 0 si l'incrementation n'a pas ete realisee
// car le maximun a ete atteint (max(0),max(1),...,max(nx-1)
int nisp_incrementation(int k[], int min[], int max[], int nx) {
	for(int i=0;i<nx;i++) {
		if(k[i]<max[i]) {
			k[i]+=1;
			for(int j=0;j<i;j++) {
				k[j]=min[j];
			}
			return(1);
		}
	}
	return(0);
}

// Norme L1 du multi indice
int norme_L1(int k[], int nx) {
	int i,s=0;
	for(i=0;i<nx;i++) s+=k[i];
	return(s);
}

//Norme Linfini du multi indice
int norme_Linf(int k[], int nx) {
	int i,s;
	for(s=k[0],i=1;i<nx;i++) {
		if(k[i]>s) {
			s=k[i];
		}
	}
	return(s);
}

// Filtre : norme min <= L1 <= max
int Filtre_norme_L1(int k[], int nx, int vmin, int vmax) {
	int norme=norme_L1(k,nx);
	if((norme>=vmin) && (norme<=vmax)) {
		return(1);
	}
	else {
		return(0);
	}
}

int Cnp(int n, int p) {
	if(n<p) {
		ostringstream msg;
		msg<<"NISP - ERROR"<<endl;
		msg << "Nisp(Cnp) : n =" << n << " < p=" << p << " !! " << endl;
		nisp_error ( msg.str() );
		return(0);
	}
	if(p<0) {
		ostringstream msg;
		msg<<"NISP - ERROR"<<endl;
		msg << "Nisp(Cnp) : p=" << p << " < 0 !! " << endl;
		nisp_error ( msg.str() );
		return(0);
	}
	int i,fn,fd;
	// factoriel n sur factoriel p
	for(fn=1,i=p+1;i<=n;i++) {
		fn=fn*i;
	}
	// factoriel n-p
	for(fd=1,i=1;i<=(n-p);i++) {
		fd=fd*i;
	}
	return(fn/fd);
}

// Constition de l'ensemble des multi indices
// A l'etat initial *indices est un vecteur nul
// A l'etat final *indices contient tous les multi indices k
// qui verifient la contrainte vmin <= norme_L1(k) <= vmax
void nisp_Build_MultiIndex_Filtre_norme_L1(vector<int *> * indices, int * min, int * max, int nx, int vmin, int vmax) {
	int * k = ivector(nx);
	int * knew;
	int i;

	for(i=0;i<nx;i++) k[i]=min[i];
	if(Filtre_norme_L1(k,nx,vmin,vmax)) {
		knew = ivector(nx);
		for(i=0;i<nx;i++) {
			knew[i]=k[i];
		}
		indices->push_back(knew);
	}
	while (nisp_incrementation(k,min,max,nx)) {
		if(Filtre_norme_L1(k,nx,vmin,vmax)) {
			knew = ivector(nx);
			for(i=0;i<nx;i++) {
				knew[i]=k[i];
			}
			indices->push_back(knew);
		}
	}
	free_ivector(k);
}

void nisp_Compute_Coef_Cnp_Signe(int *coef_Cnp, int *signe, vector<int *> * indices, int nx, int l) {
	vector<int *>::iterator it;
	int norme_L1(int *k, int nx);
	int *k, norme, j=0;
	for(it=indices->begin();it!=indices->end();++it) {
		k=*it;
		norme = norme_L1(k,nx);
		coef_Cnp[j]= Cnp(nx-1,norme-l);
		if((l+nx-norme-1)%2) {
			signe[j]=-1;
		}
		else {
			signe[j]=1;
		}
		j++;
	}
}

