//
// Copyright (C)  2006-2009 - CEA - Jean-Marc Martinez
// Copyright (C)  2009 - INRIA - Michael Baudin
// Copyright (C)  2009 - Digiteo - Michael Baudin
//
// This file is released under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html
//
#include <iostream>
#include <sstream>
#include <fstream>
#include <string>
#include <vector>
#include <math.h>

#include "dcdflib.h"
#include "nisp_va.h"
#include "nisp_util.h"
#include "nisp_msg.h"
#include "nisp_inv.h"

using namespace std;

// ===========
// Methodes RandomVariable
// ===========
void RandomVariable::SetDefaultParameters() {
	if     (type=="Normale")       {
		a=0.;  
		b=1.;
	}
	else if(type=="Uniforme")      {
		a=0.;  
		b=1.;
	}
	else if(type=="Exponentielle") {
		a=1.;  
		b=0.; 
	}
	else if(type=="LogNormale")    {
		a=0.1; 
		b=1.;
	}
	else if(type=="LogUniforme")   {
		a=0.1; 
		b=1.;
	}
	else  {
		ostringstream msg;
		msg<<"NISP - ERROR"<<endl;
		msg << "Nisp(RandomVariable::RandomVariable) : law " << type << " unknown" << endl;
		nisp_error ( msg.str() );
	}
}
void RandomVariable::AssignFunction() {
	ostringstream msg;
	if (type=="Normale") {
		function = nisp_normale;
	}
	else if(type=="Uniforme") {
		function = nisp_uniforme;
	}
	else if(type=="LogNormale") {
		function = nisp_lognormale;
	}
	else if(type=="LogUniforme")   {
		function = nisp_loguniforme;
	}
	else if (type=="Exponentielle") {
		function = nisp_exponentielle;
	}
	else {
		ostringstream msg;
		msg<<"NISP - ERROR"<<endl;
		msg << "Nisp(RandomVariable::RandomVariable) : unknown law " << type << " with 2 parameters" << endl;
		nisp_error ( msg.str() );
	}
}
void RandomVariable::CheckParameters() {
	ostringstream msg;
	if (type=="Normale") {
		if(b<0) {
			msg<<"NISP - ERROR"<<endl;
			msg << "Nisp(RandomVariable::RandomVariable) : law " << type << " and standard error " << b << " < 0 " << endl;
			nisp_error ( msg.str() );
		}
	}
	else if(type=="Uniforme") {
		if(a>=b) {
			msg<<"NISP - ERROR"<<endl;
			msg << "Nisp(RandomVariable::RandomVariable) : law " << type << " and min = " << a << " >= max = " << b << endl;
			nisp_error ( msg.str() );
		}
	}
	else if(type=="LogNormale") {
		if((a<=0.) || (b<=0.)) {
			msg<<"NISP - ERROR"<<endl;
			msg << "Nisp(RandomVariable::RandomVariable) : law " << type << " and one of parameters are " << a << " , " << b << " <=0" << endl;
			nisp_error ( msg.str() );
		}
	}
	else if(type=="LogUniforme")   {
		if((a<=0.) || (b<=0.)) {
			msg<<"NISP - ERROR"<<endl;
			msg << "Nisp(RandomVariable::RandomVariable) : law " << type << " and one of parameters are " << a << " , " << b << " <=0" << endl;
			nisp_error ( msg.str() );
		}
		if(a>b) {
			msg<<"NISP - ERROR"<<endl;
			msg << "Nisp(RandomVariable::RandomVariable) : law " << type << " and min = " << a << " > max = " << b << endl;
			nisp_error ( msg.str() );
		}
	}
	else if (type=="Exponentielle") {
		// Nothing to do
	}
	else {
		ostringstream msg;
		msg<<"NISP - ERROR"<<endl;
		msg << "Nisp(RandomVariable::RandomVariable) : unknown law " << type << " with 2 parameters" << endl;
		nisp_error ( msg.str() );
	}
}
RandomVariable::RandomVariable(string name) {
	type=name;
	SetDefaultParameters();
	CheckParameters();
	AssignFunction();
}
RandomVariable::RandomVariable(char * name) {
	type.assign(name);
	SetDefaultParameters();
	CheckParameters();
	AssignFunction();
}
RandomVariable::~RandomVariable() {
	// Nothing to do for now
}
RandomVariable::RandomVariable(string name, double x) {
	type=name; 
	a=x; 
	b=0.;
	CheckParameters();
	AssignFunction();
}
RandomVariable::RandomVariable(char * name, double x) {
	type.assign(name); 
	a=x; 
	b=0.;
	CheckParameters();
	AssignFunction();
}
RandomVariable::RandomVariable(string name, double x, double y) { 
	type=name; 
	a=x; 
	b=y;
	CheckParameters();
	AssignFunction();
}
RandomVariable::RandomVariable(char * name, double x, double y) {
	type.assign(name);
	a=x; 
	b=y;
	CheckParameters();
	AssignFunction();
}
double RandomVariable::GetValue() {
	return(function(a,b));
}
double RandomVariable::GetValue(RandomVariable *mere, double x) {
	double value;
	value = this->pdfChange(mere,x);
	return(value);
}
void RandomVariable::GetLog() {
	ostringstream msg;
	msg << "***********************************************" << endl;
	msg << "Nisp(RandomVariable::GetLog) for RandomVariable" << endl;
	msg << type << " : " << a << " : " << b << endl;
	msg << "***********************************************" << endl;
	nisp_message ( msg.str() );
}


// mere est uniforme dans [0,1]
double RandomVariable::pdfChange ( double z) {
	double y;
	double a=this->a; 
	double b=this->b;
	string typefils=this->type;
	if((z<0.) || (z>1.)) {
		ostringstream msg;
		msg<<"NISP - ERROR"<<endl;
		msg << "Nisp(pdfChange) : value " << z << "is not in [0,1]" << endl;
		nisp_error ( msg.str() );
		return(0.);
	}
	if(typefils=="Uniforme") {
		y=a+(b-a)*z;
	}
	else if(typefils=="Normale") {
		y=a+b*nisp_invfcn(z);
	}
	else if(typefils=="Exponentielle") {
		y=(-1./a)*log(z);
	}
	else if(typefils=="LogNormale") {
		y=exp(log(a)-b*b/2.+b*nisp_invfcn(z));
	}
	else if(typefils=="LogUniforme") {
		y=exp(log(a)+log(b/a)*z);
	}
	else 
	{
		ostringstream msg;
		msg<<"NISP - ERROR"<<endl;
		msg << "Nisp(pdfChange) : law " << typefils << " unknown" << endl;
		nisp_error ( msg.str() );
		return(0.);
	}
	return(y);
}
double RandomVariable::pdfChange ( RandomVariable *mere, double x) {
	// this = fils
	double y;
	double a=this->a; 
	double b=this->b;
        int ind;
        double xerf;
	string typefils=this->type; 
	string typemere=mere->type;
	ostringstream msg;
	if(typemere=="Normale") 
	{            // Mere est Normale
		double mean=mere->a; 
		double sigma=mere->b;
		double z=(x-mean)/sigma;           // z suit une loi normale centree reduite
		if(typefils=="Normale") {
			y=a+b*z;
		}
		else if(typefils=="Uniforme") {
                        ind = 0;
                        xerf = -z/sqrt(2.);
			y=a+(b-a)*(error_fc( &ind , &xerf )/2.);
		}
		else if(typefils=="Exponentielle") {
                        ind = 0;
                        xerf = -z/sqrt(2.);
			y=-(1./a)*log((error_fc( &ind , &xerf )/2.));
		}
		else if(typefils=="LogNormale") {
			y=exp(log(a)-b*b/2.+b*z);
		}
		else if(typefils=="LogUniforme") {
                        ind = 0;
                        xerf = -z/sqrt(2.);
			y=exp(log(a)+log(b/a)*(error_fc( &ind , &xerf )/2.));
		}
		else 
		{
			msg<<"NISP - ERROR"<<endl;
			msg << "Nisp(pdfChange) : Transformation Normale -> " << typefils << " not available" << endl;
			nisp_error ( msg.str() );
			return(0.);
		}
	}
	else if(typemere=="LogNormale") 
	{    // Mere est LogNormale
		if(x<=0.) {
			msg<<"NISP - ERROR"<<endl;
			msg << "Nisp(pdfChange) : law (mother)is LogNormale and value " << x << " < 0" << endl;
			nisp_error ( msg.str() );
			return(0.);
		}
		double mean=mere->a; 
		double sigma=mere->b; 
		double mu=log(mean)-sigma*sigma/2.;
		double z=(log(x)-mu)/sigma;        // z suit une loi normale centree reduite
		if(typefils=="Normale") {
			y=a+b*z;
		}
		else if(typefils=="Uniforme") {
                        ind = 0;
                        xerf = -z/sqrt(2.);
			y=a+(b-a)*(error_fc( &ind , &xerf )/2.);
		}
		else if(typefils=="Exponentielle") {
                        ind = 0;
                        xerf = -z/sqrt(2.);
			y=-(1./a)*log((error_fc( &ind , &xerf )/2.));
		}
		else if(typefils=="LogNormale") {
			y=exp(log(a)-b*b/2.+b*z);
		}
		else if(typefils=="LogUniforme") {
                        ind = 0;
                        xerf = -z/sqrt(2.);
			y=exp(log(a)+log(b/a)*(error_fc( &ind , &xerf )/2.));
		}
		else 
		{
			msg<<"NISP - ERROR"<<endl;
			msg << "Nisp(pdfChange) : Transformation LogNormale -> " << typefils << " not available" << endl;
			nisp_error ( msg.str() );
			return(0.);
		}
	}
	else if(typemere=="Uniforme") 
	{      // Mere est Uniforme
		double min=mere->a; 
		double max=mere->b;
		if((x<min) || (x>max)) {
			msg<<"NISP - ERROR"<<endl;
			msg << "Nisp(pdfChange) : value " << x << " must be in ["<<min<<","<<max<<"]"<< endl;
			nisp_error ( msg.str() );
			return(0.);
		}
		double z=(x-min)/(max-min);        // z suit une loi uniforme [0,1]
		if(typefils=="Uniforme") {
			y=a+(b-a)*z;
		}
		else if(typefils=="Normale") {
			y=a+b*nisp_invfcn(z);
		}
		else if(typefils=="Exponentielle") {
			y=(-1./a)*log(z);
		}
		else if(typefils=="LogNormale") {
			y=exp(log(a)-b*b/2.+b*nisp_invfcn(z));
		}
		else if(typefils=="LogUniforme") {
			y=exp(log(a)+log(b/a)*z);
		}
		else 
		{
			msg<<"NISP - ERROR"<<endl;
			msg << "Nisp(pdfChange) : Transformation Uniform -> " << typefils << " not available" << endl;
			nisp_error ( msg.str() );
			return(0.);
		}
	}
	else if(typemere=="LogUniforme") 
	{                // Mere est LogUniforme
		double min=mere->a; 
		double max=mere->b;
		if(x<=0.) {
			msg<<"NISP - ERROR"<<endl;
			msg << "Nisp(pdfChange) : law (mother)is LogUniform and value " << x << " < 0" << endl;
			nisp_error ( msg.str() );
			return(0.);
		}
		double z=(log(x)-log(min))/(log(max)-log(min)); // z suit une loi uniforme [0,1]
		if((z<0.) || (z>1.)) {
			msg<<"NISP - ERROR"<<endl;
			msg << "Nisp(pdfChange) : law (mother) is LogUniform and error on value " << x << endl;
			nisp_error ( msg.str() );
			return(0.);
		}
		if(typefils=="Uniforme") {
			y=a+(b-a)*z;
		}
		else if(typefils=="Normale") {
			y=a+b*nisp_invfcn(z);
		}
		else if(typefils=="Exponentielle") {
			y=(-1./a)*log(z);
		}
		else if(typefils=="LogNormale") {
			y=exp(log(a)-b*b/2.+b*nisp_invfcn(z));
		}
		else if(typefils=="LogUniforme") {
			y=exp(log(a)+log(b/a)*z);
		}
		else 
		{
			msg<<"NISP - ERROR"<<endl;
			msg << "Nisp(pdfChange) : Transformation LogUniform -> " << typefils << " not available" << endl;
			nisp_error ( msg.str() );
			return(0.);
		}
	}
	else if(typemere=="Exponentielle") 
	{ // Mere est Exponentielle
		double lambda=mere->a;
		if(x<=0.) {
			msg<<"NISP - ERROR"<<endl;
			msg << "Nisp(pdfChange) : law (mother)is Exponential and value " << x << " <= 0" << endl;
			nisp_error ( msg.str() );
			return(0.);
		}
		double z=x/lambda;                 // z suit une loi exponentielle lambda=1;
		if(typefils=="Exponentielle") {
			y=z/a;
		}
		else 
		{
			msg<<"NISP - ERROR"<<endl;
			msg << "Nisp(pdfChange) : Transformation Exponential -> " << typefils << " not available" << endl;
			nisp_error ( msg.str() );
			return(0.);
		}
	}
	else 
	{
		msg<<"NISP - ERROR"<<endl;
		msg << "Nisp(pdfChange) : law " << typemere << " unknown" << endl;
		nisp_error ( msg.str() );
		return(0.);
	}
	return(y);
}

