
// Copyright (C) 2008-2009 - INRIA - Michael Baudin
// Copyright (C) 2010 - INRIA - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html

//
// assert_close --
//   Returns 1 if the two real matrices computed and expected are close,
//   i.e. if the relative distance between computed and expected is lesser than epsilon.
// Arguments
//   computed, expected : the two matrices to compare
//   epsilon : a small number
//
function flag = assert_close ( computed, expected, epsilon )
  if expected==0.0 then
    shift = norm(computed-expected);
  else
    shift = norm(computed-expected)/norm(expected);
  end
  if shift < epsilon then
    flag = 1;
  else
    flag = 0;
  end
  if flag <> 1 then pause,end
endfunction

//
// assert_equal --
//   Returns 1 if the two real matrices computed and expected are equal.
// Arguments
//   computed, expected : the two matrices to compare
//   epsilon : a small number
//
function flag = assert_equal ( computed , expected )
  if computed==expected then
    flag = 1;
  else
    flag = 0;
  end
  if flag <> 1 then pause,end
endfunction

//
// <-- JVM NOT MANDATORY -->
// <-- ENGLISH IMPOSED -->

////////////////////////////////////////////////////////////////////////
// 
// Check Argument Checking
//
assert_checkerror ( "nisp_erfcinv()" , "nisp_erfcinv: Unexpected number of input arguments : 0 provided while the number of expected input arguments should be in the set [1]." );
assert_checkerror ( "nisp_erfcinv(1,2)" , "Wrong number of input arguments:" );
assert_checkerror ( "nisp_erfcinv(""a"")" , "nisp_erfcinv: Expected type [""constant""] for input argument x at input #1, but got ""string"" instead." );
////////////////////////////////////////////////////////////////////////
// 
// Check robustness
computed = nisp_erfcinv([]);
assert_checkequal ( computed , [] );
//
// IEEE values
computed = nisp_erfcinv([%inf -%inf]);
assert_checkequal ( computed , [%nan %nan] );
//
// IEEE values
computed = nisp_erfcinv([+0 -0 %nan]);
assert_checkequal ( computed , [%inf %inf %nan] );
//
// Out-of-bounds
computed = nisp_erfcinv([-1 -2 3 4]);
assert_checkequal ( computed , [%nan %nan %nan %nan] );
//
// A complex
assert_checkerror ( "nisp_erfcinv(1+%i)","nisp_erfcinv: Wrong type for input argument #1: Real matrix expected.");
//
// A row vector
computed = nisp_erfcinv([0.1 0.2 0.3 0.9]);
expected = [
  1.163087153676674   
  0.906193802436823   
  0.732869077959217
  0.088855990494258
]';
assert_checkalmostequal ( computed , expected , 10*%eps );
//
// A column vector
computed = nisp_erfcinv([0.1 0.2 0.3 0.9]');
expected = [
  1.163087153676674   
  0.906193802436823   
  0.732869077959217
  0.088855990494258
];
assert_checkalmostequal ( computed , expected , 10*%eps );
//
// A matrix
computed = nisp_erfcinv(0.5*ones(2,3));
expected = 0.476936276204470 * ones(2,3);
assert_checkalmostequal ( computed , expected , 10*%eps );
////////////////////////////////////////////////////////////
//
// Some simple accuracy tests
// [x fexact digits]
table = [
1.e-3     2.326753765513525        15
1.e-6     3.458910737279500        15
1.e-9     4.320005384913445        15
1.e-12    5.042029745639059        15
1.e-20    6.601580622355143        15
1-1.e-5   8.862269254759593803e-6  10
1-1.e-10  8.862269254527580136e-11  6
1-1.e-15  8.862269254527580136e-16  0
1+1.e-15 -8.862269254527580136e-16  0
1+1.e-10 -8.862269254527580136e-11  6
1+1.e-5  -8.862269254759593803e-6  10
2-1.e-3  -2.326753765513533        14
2-1.e-6  -3.458910737279500        11
2-1.e-9  -4.320005384913445         8
2-1.e-12 -5.04202974563905937       5
2        -%inf                     15
];
ntests = size(table,"r");
x = table(:,1);
computed = nisp_erfcinv(x);
expected = table(:,2);
digits = assert_computedigits ( computed , expected );
requireddigits = table(:,3);
assert_checktrue ( digits >= requireddigits );
for i = 1 : ntests
  if ( %f ) then
    mprintf("Test #%2d/%2d, x=%s, y=%s, e=%s, d=%3.1f, %s\n", ..
      i, ntests, sci2exp(x(i)), sci2exp(computed(i)), sci2exp(expected(i)) , digits(i), ok);
  end
end




