
// Copyright (C) 2008-2009 - INRIA - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html

//
// assert_close --
//   Returns 1 if the two real matrices computed and expected are close,
//   i.e. if the relative distance between computed and expected is lesser than epsilon.
// Arguments
//   computed, expected : the two matrices to compare
//   epsilon : a small number
//
function flag = assert_close ( computed, expected, epsilon )
  if expected==0.0 then
    shift = norm(computed-expected);
  else
    shift = norm(computed-expected)/norm(expected);
  end
  if shift < epsilon then
    flag = 1;
  else
    flag = 0;
  end
  if flag <> 1 then pause,end
endfunction

//
// assert_equal --
//   Returns 1 if the two real matrices computed and expected are equal.
// Arguments
//   computed, expected : the two matrices to compare
//   epsilon : a small number
//
function flag = assert_equal ( computed , expected )
  if computed==expected then
    flag = 1;
  else
    flag = 0;
  end
  if flag <> 1 then pause,end
endfunction

// Try randvar_tokens
// At start, no randvar should exist.
rvlist = randvar_tokens();
assert_equal ( rvlist , [] );
nbrv = randvar_size();
assert_equal ( nbrv , 0 );

// The simplest possible test.
rv = randvar_new ( "Exponentielle" );
randvar_destroy(rv);

// Create two randvars and check them
// Test randvar_size, randvar_new, randvar_destroy
rv1 = randvar_new("Normale",1.0,0.5);
rv2 = randvar_new("Uniforme",1.0,2.5);
nbrv = randvar_size();
assert_equal ( nbrv , 2 );
rvlist = randvar_tokens();
assert_equal ( size(rvlist,2) , 2 );
randvar_destroy(rv1);
randvar_destroy(rv2);
rvlist = randvar_tokens();
assert_equal ( rvlist , [] );
nbrv = randvar_size();
assert_equal ( nbrv , 0 );

// Check that the size is zero
nbrv = randvar_size();
assert_equal ( nbrv , 0 );
rvlist = randvar_tokens();
assert_equal ( rvlist , [] );

// Try randvar_getlog
rv = randvar_new("Normale",1.0,0.5);
randvar_getlog(rv);
randvar_destroy(rv);

// Try randvar_getvalue on Normal law and 2 parameters
nisp_initseed ( 0 );
mu = 1.0;
sigma = 0.5;
rv = randvar_new("Normale" , mu , sigma);
nbshots = 1000;
for i=1:nbshots
  values(i) = randvar_getvalue(rv);
end
computed = mean (values);
expected = mu;
assert_close ( computed , expected , 1.e-1 );
computed = variance (values);
expected = sigma^2;
assert_close ( computed , expected , 1.e-1 );
randvar_destroy(rv);

// Try randvar_getvalue on Normal law and default parameters
nisp_initseed ( 0 );
rv = randvar_new("Normale");
nbshots = 1000;
for i=1:nbshots
  values(i) = randvar_getvalue(rv);
end
mu = 0.0;
sigma = 1.0;
computed = mean (values);
expected = mu;
assert_close ( computed , expected , 1.e-1 );
computed = variance (values);
expected = sigma^2;
assert_close ( computed , expected , 1.e-1 );
randvar_destroy(rv);

//
// Test with Uniform law and 2 parameters
//
nisp_initseed ( 0 );
a= 2;
b = 3;
rv = randvar_new("Uniforme" , a , b );
nbshots = 1000;
for i=1:nbshots
  values(i) = randvar_getvalue(rv);
end
computed = mean (values);
expected = (a+b)/2;
assert_close ( computed , expected , 1.e-1 );
computed = variance (values);
expected = (b-a)^2/12;
assert_close ( computed , expected , 1.e-1 );
randvar_destroy(rv);

//
// Test with Uniform law and default parameter
//
nisp_initseed ( 0 );
rv = randvar_new("Uniforme");
nbshots = 1000;
for i=1:nbshots
  values(i) = randvar_getvalue(rv);
end
a = 0.0;
b = 1.0;
computed = mean (values);
expected = (a+b)/2;
assert_close ( computed , expected , 1.e-1 );
computed = variance (values);
expected = (b-a)^2/12;
assert_close ( computed , expected , 1.e-1 );
randvar_destroy(rv);

// What if the law does not exist ?
// TODO : problem : the unit test system does not allow to test error cases.
//rv1 = randvar_new("Foo",2.0,3.0);

// Test Exponentielle with 1 parameter
nisp_initseed ( 0 );
lambda = 2.0;
rv = randvar_new("Exponentielle" , lambda );
nbshots = 1000;
for i=1:nbshots
  values(i) = randvar_getvalue(rv);
end
computed = mean (values);
expected = 1/lambda;
assert_close ( computed , expected , 1.e-1 );
computed = variance (values);
expected = 1/lambda^2;
assert_close ( computed , expected , 1.e-1 );
randvar_destroy(rv);

// Test Exponentielle with default parameter
nisp_initseed ( 0 );
rv = randvar_new("Exponentielle");
nbshots = 1000;
for i=1:nbshots
  values(i) = randvar_getvalue(rv);
end
lambda = 1.0;
computed = mean (values);
expected = 1/lambda;
assert_close ( computed , expected , 1.e-1 );
computed = variance (values);
expected = 1/lambda^2;
assert_close ( computed , expected , 1.e-1 );
randvar_destroy(rv);

// Test LogNormale with 2 parameters
nisp_initseed ( 0 );
mu = 1.0;
sigma = 2.0;
rv = randvar_new("LogNormale",mu,sigma);
nbshots = 1000;
for i=1:nbshots
  values(i) = randvar_getvalue(rv);
end
computed = mean (values);
expected = mu;
assert_close ( computed , expected , 2.e-1 );
computed = variance (values);
// Compute the mun parameter of the Normal law : mu = exp(mun+0.5*sigma^2)
mun = log(mu) - 0.5*sigma^2;
expected = ( exp(sigma^2) - 1.0 ) * exp ( 2*mun + sigma^2 );
assert_close ( computed , expected , 9.e-1 );
randvar_destroy(rv);

// Test LogNormale with default parameters
nisp_initseed ( 0 );
rv = randvar_new("LogNormale");
nbshots = 1000;
for i=1:nbshots
  values(i) = randvar_getvalue(rv);
end
mu = 0.1;
sigma = 1.0;
computed = mean (values);
expected = mu;
assert_close ( computed , expected , 1.e-1 );
computed = variance (values);
// Compute the mun parameter of the Normal law : mu = exp(mun+0.5*sigma^2)
mun = log(mu) - 0.5*sigma^2;
expected = ( exp(sigma^2) - 1.0 ) * exp ( 2*mun + sigma^2 );
assert_close ( computed , expected , 2.e-1 );
randvar_destroy(rv);

// Test LogUniforme with 2 parameters
nisp_initseed ( 0 );
a = 10.0;
b = 20.0;
rv = randvar_new("LogUniforme",a,b);
nbshots = 1000;
for i=1:nbshots
  values(i) = randvar_getvalue(rv);
end
computed = mean (values);
mu = (b-a)/(log(b)-log(a));
expected = mu;
assert_close ( computed , expected , 1.e-1 );
computed = variance (values);
expected = 0.5 * ( b^2 - a^2)/(log(b) - log(a)) - mu^2;
assert_close ( computed , expected , 1.e-1 );
randvar_destroy(rv);

// Test LogUniforme with default parameters
nisp_initseed ( 0 );
rv = randvar_new("LogUniforme");
nbshots = 1000;
for i=1:nbshots
  values(i) = randvar_getvalue(rv);
end
a = 0.1;
b = 1.0;
computed = mean (values);
mu = (b-a)/(log(b)-log(a));
expected = mu;
assert_close ( computed , expected , 1.e-1 );
computed = variance (values);
expected = 0.5 * ( b^2 - a^2)/(log(b) - log(a)) - mu^2;
assert_close ( computed , expected , 1.e-1 );
randvar_destroy(rv);

//-----------------------------------------
//
// Test transformations
//

//
// Uniform -> LogUniforme
// (Transform a Uniform outcome into a LogUniforme outcome)
//
nisp_initseed ( 0 );
a = 10.0;
b = 20.0;
rv = randvar_new ( "LogUniforme" , a , b );
rv2 = randvar_new ( "Uniforme" , 2 , 3 );
nbshots = 1000;
for i=1:nbshots
  value2 = randvar_getvalue( rv2 );
  values(i) = randvar_getvalue( rv , rv2 , value2 );
end
// Check that the values are from a LogUniforme law with parameters (a,b)
computed = mean (values);
mu = (b-a)/(log(b)-log(a));
expected = mu;
assert_close ( computed , expected , 1.e-1 );
randvar_destroy(rv);
randvar_destroy(rv2);

//
// Uniform -> Uniform
//
nisp_initseed ( 0 );
a = 10.0;
b = 20.0;
rv = randvar_new ( "Uniforme" , a , b );
rv2 = randvar_new ( "Uniforme" , 2 , 3 );
nbshots = 1000;
for i=1:nbshots
  value2 = randvar_getvalue( rv2 );
  values(i) = randvar_getvalue( rv , rv2 , value2 );
end
// Check that the values are from a Uniforme law with parameters (a,b)
computed = mean (values);
mu = (a+b)/2;
expected = mu;
assert_close ( computed , expected , 1.e-1 );
randvar_destroy(rv);
randvar_destroy(rv2);

//
// Uniform -> Normale
//
nisp_initseed ( 0 );
mu = 10.0;
sigma = 20.0;
rv = randvar_new ( "Normale" , mu , sigma );
rv2 = randvar_new ( "Uniforme" , 2 , 3 );
nbshots = 1000;
for i=1:nbshots
  value2 = randvar_getvalue( rv2 );
  values(i) = randvar_getvalue( rv , rv2 , value2 );
end
// Check that the values are from a Uniforme law with parameters (a,b)
computed = mean (values);
expected = mu;
assert_close ( computed , expected , 2.e-1 );
randvar_destroy(rv);
randvar_destroy(rv2);

// TODO : Uniforme > "Exponentielle"
// TODO : Uniforme > "LogNormale"
// TODO : Normale > *
// TODO : LogNormale > *
// TODO : LogUniforme > *
// TODO : Exponentielle > Exponentielle

//
// TODO : what happens when an error is committed ?
// rv2 = randvar_new("LogNormale",-12.0,-12.0);
//

