
// Copyright (C) 2008-2009 - INRIA - Michael Baudin
// Copyright (C) 2010 - INRIA - Michael Baudin
//
// This file must be used under the terms of the GNU Lesser General Public License license :
// http://www.gnu.org/copyleft/lesser.html

//
// assert_close --
//   Returns 1 if the two real matrices computed and expected are close,
//   i.e. if the relative distance between computed and expected is lesser than epsilon.
// Arguments
//   computed, expected : the two matrices to compare
//   epsilon : a small number
//
function flag = assert_close ( computed, expected, epsilon )
  if expected==0.0 then
    shift = norm(computed-expected);
  else
    shift = norm(computed-expected)/norm(expected);
  end
  if shift < epsilon then
    flag = 1;
  else
    flag = 0;
  end
  if flag <> 1 then pause,end
endfunction

//
// assert_equal --
//   Returns 1 if the two real matrices computed and expected are equal.
// Arguments
//   computed, expected : the two matrices to compare
//   epsilon : a small number
//
function flag = assert_equal ( computed , expected )
  if computed==expected then
    flag = 1;
  else
    flag = 0;
  end
  if flag <> 1 then pause,end
endfunction

// Try setrandvar_tokens
// At start, no randvar should exist.
srvlist = setrandvar_tokens();
assert_equal ( srvlist , [] );
nbsrv = setrandvar_size();
assert_equal ( nbsrv , 0 );

// The simplest possible test.
srv = setrandvar_new ( );
setrandvar_destroy(srv);

// Check that multiple destroy generate an error : cannot test this !
srv = setrandvar_new ( );
setrandvar_destroy(srv);
//setrandvar_destroy(srv);

// Create two randvars and check them
// Test setrandvar_size, setrandvar_new, setrandvar_destroy
srv1 = setrandvar_new();
srv2 = setrandvar_new();
nbsrv = setrandvar_size();
assert_equal ( nbsrv , 2 );
srvlist = setrandvar_tokens();
assert_equal ( size(srvlist,2) , 2 );
setrandvar_destroy(srv1);
setrandvar_destroy(srv2);
nbsrv = setrandvar_size();
assert_equal ( nbsrv , 0 );

// Create a set of 5 Uniform random variables
srv1 = setrandvar_new ( 5 );
setrandvar_destroy(srv1);

// Create a set random variables from a file
// TODO : what should be the content of the file ?
//srv1 = setrandvar_new ( "setrandvar.data" );
//setrandvar_destroy(srv1);

// Test freememory
srv1 = setrandvar_new ( 5 );
setrandvar_freememory ( srv1 );
setrandvar_destroy(srv1);

// Test setrandvar_addrandvar
srv = setrandvar_new();
rv1 = randvar_new("Normale",1.0,0.5);
rv2 = randvar_new("Uniforme",1.0,2.5);
setrandvar_addrandvar ( srv , rv1 );
setrandvar_addrandvar ( srv , rv2 );
setrandvar_destroy(srv);
randvar_destroy(rv1);
randvar_destroy(rv2);

// Test setrandvar_getlog
srv = setrandvar_new ( );
rv1 = randvar_new("Normale",1.0,0.5);
rv2 = randvar_new("Uniforme",1.0,2.5);
setrandvar_addrandvar ( srv , rv1 );
setrandvar_addrandvar ( srv , rv2 );
setrandvar_getlog ( srv );
setrandvar_destroy(srv);
randvar_destroy(rv1);
randvar_destroy(rv2);

// Test setrandvar_getdimension
srv = setrandvar_new ( );
rv1 = randvar_new("Normale",1.0,0.5);
rv2 = randvar_new("Uniforme",1.0,2.5);
setrandvar_addrandvar ( srv , rv1 );
setrandvar_addrandvar ( srv , rv2 );
d = setrandvar_getdimension ( srv );
assert_equal ( d , 2 );
setrandvar_destroy(srv);
randvar_destroy(rv1);
randvar_destroy(rv2);

// Test setrandvar_getsize, when there is no DOE
srv = setrandvar_new ( );
rv1 = randvar_new("Normale",1.0,0.5);
rv2 = randvar_new("Uniforme",1.0,2.5);
setrandvar_addrandvar ( srv , rv1 );
setrandvar_addrandvar ( srv , rv2 );
s = setrandvar_getsize ( srv );
assert_equal ( s , 0 );
setrandvar_destroy(srv);
randvar_destroy(rv1);
randvar_destroy(rv2);

// Test setrandvar_getsample, when there is 
// no DOE > an error is generated, which cannot be tested.
srv = setrandvar_new ( );
rv1 = randvar_new("Normale",1.0,0.5);
rv2 = randvar_new("Uniforme",1.0,2.5);
setrandvar_addrandvar ( srv , rv1 );
setrandvar_addrandvar ( srv , rv2 );
//s = setrandvar_getsample ( srv , 12 , 13 );
setrandvar_destroy(srv);
randvar_destroy(rv1);
randvar_destroy(rv2);

// Test setrandvar_setsample with "foo" sampling and 12 simulations
// Set the sample twice to see if and error is generated so that no memory is lost.
srv = setrandvar_new ( );
rv1 = randvar_new("Normale",1.0,0.5);
rv2 = randvar_new("Uniforme",1.0,2.5);
setrandvar_addrandvar ( srv , rv1 );
setrandvar_addrandvar ( srv , rv2 );
np = 12;
setrandvar_setsample ( srv , "foo" , np );
// The second call makes NISP generate an error
// setrandvar_setsample ( srv , "foo" , np );
setrandvar_destroy(srv);
randvar_destroy(rv1);
randvar_destroy(rv2);

// Test setrandvar_setsample with "foo" sampling and 12 simulations
// test setrandvar_setsample / setrandvar_getsample with a scalar
srv = setrandvar_new ( );
rv1 = randvar_new("Normale",1.0,0.5);
rv2 = randvar_new("Uniforme",1.0,2.5);
setrandvar_addrandvar ( srv , rv1 );
setrandvar_addrandvar ( srv , rv2 );
np = 12;
setrandvar_setsample ( srv , "foo" , np );
for k = 1 : np
  for i = 1 : 2
    value = k+i;
    setrandvar_setsample ( srv , k , i , value );
  end
end
for k = 1 : np
  for i = 1 : 2
    computed = setrandvar_getsample ( srv , k , i );
    expected = k+i;
    assert_close ( computed , expected , %eps);
  end
end
setrandvar_destroy(srv);
randvar_destroy(rv1);
randvar_destroy(rv2);

// Test setrandvar_getsample with a matrix / a vector
srv = setrandvar_new ( );
rv1 = randvar_new("Normale",1.0,0.5);
rv2 = randvar_new("Uniforme",1.0,2.5);
setrandvar_addrandvar ( srv , rv1 );
setrandvar_addrandvar ( srv , rv2 );
np = 12;
setrandvar_setsample ( srv , "foo" , np );
for k = 1 : np
  for i = 1 : 2
    setrandvar_setsample ( srv , k , i , k+i );
  end
end
for k = 1 : np
  // Get the sample for experiment #k
  computed = setrandvar_getsample ( srv , k );
  expected = k + [1:2];
  assert_close ( computed , expected , %eps);
end
// Get the sampling matrix
computed = setrandvar_getsample ( srv );
expected = [
2 3
3 4
4 5
5 6
6 7
7 8
8 9
9 10
10 11
11 12
12 13
13 14
];
assert_close ( computed , expected , %eps);
setrandvar_destroy(srv);
randvar_destroy(rv1);
randvar_destroy(rv2);

// Test setrandvar_setsample with vector
srv = setrandvar_new ( );
rv1 = randvar_new("Normale",1.0,0.5);
rv2 = randvar_new("Uniforme",1.0,2.5);
setrandvar_addrandvar ( srv , rv1 );
setrandvar_addrandvar ( srv , rv2 );
np = 12;
setrandvar_setsample ( srv , "foo" , np );
expected = [123 456];
setrandvar_setsample ( srv , 6 , expected );
computed = setrandvar_getsample ( srv , 6 );
assert_close ( computed , expected , %eps);
setrandvar_destroy(srv);
randvar_destroy(rv1);
randvar_destroy(rv2);

// Test setrandvar_setsample with matrix
srv = setrandvar_new ( );
rv1 = randvar_new("Normale",1.0,0.5);
rv2 = randvar_new("Uniforme",1.0,2.5);
setrandvar_addrandvar ( srv , rv1 );
setrandvar_addrandvar ( srv , rv2 );
np = 12;
setrandvar_setsample ( srv , "foo" , np );
expected = [
2 3
3 4
4 5
5 6
6 7
7 8
8 9
9 10
10 11
11 12
12 13
13 14
];
setrandvar_setsample ( srv , expected );
computed = setrandvar_getsample ( srv );
assert_close ( computed , expected , %eps);
setrandvar_destroy(srv);
randvar_destroy(rv1);
randvar_destroy(rv2);

// Test setrandvar_save into data file
srv = setrandvar_new ( );
rv1 = randvar_new("Normale",1.0,0.5);
rv2 = randvar_new("Uniforme",1.0,2.5);
setrandvar_addrandvar ( srv , rv1 );
setrandvar_addrandvar ( srv , rv2 );
np = 12;
setrandvar_setsample ( srv , "foo" , np );
expected = [
2 3
3 4
4 5
5 6
6 7
7 8
8 9
9 10
10 11
11 12
12 13
13 14
];
setrandvar_setsample ( srv , expected );
setrandvar_save ( srv , TMPDIR+"/mydata.txt" );
setrandvar_destroy(srv);
randvar_destroy(rv1);
randvar_destroy(rv2);
// Now create a new set, based on the file.
srv = setrandvar_new ( TMPDIR+"/mydata.txt" );
computed = setrandvar_getsample ( srv );
assert_close ( computed , expected , %eps);
setrandvar_destroy(srv);

// Test setrandvar_buildsample with ( name , np ) and with MonteCarlo
nisp_initseed ( 0 );
srv = setrandvar_new ( );
rv1 = randvar_new("Normale",1.0,0.5);
rv2 = randvar_new("Uniforme",1.0,2.5);
setrandvar_addrandvar ( srv , rv1 );
setrandvar_addrandvar ( srv , rv2 );
np = 1000;
name = "MonteCarlo";
setrandvar_buildsample ( srv , name , np );
sampling = setrandvar_getsample ( srv );
// Check sampling of random variable #1
computed = mean(sampling(1:np,1));
expected = 1.0;
assert_close ( computed , expected , 1.e-1 );
// Check sampling of random variable #2
computed = mean(sampling(1:np,2));
expected = 1.75;
assert_close ( computed , expected , 1.e-1 );
setrandvar_destroy(srv);
randvar_destroy(rv1);
randvar_destroy(rv2);

// Test setrandvar_buildsample with ( name , np ) and with Lhs
nisp_initseed ( 0 );
srv = setrandvar_new ( );
rv1 = randvar_new("Normale",1.0,0.5);
rv2 = randvar_new("Uniforme",1.0,2.5);
setrandvar_addrandvar ( srv , rv1 );
setrandvar_addrandvar ( srv , rv2 );
np = 1000;
name = "Lhs";
setrandvar_buildsample ( srv , name , np );
sampling = setrandvar_getsample ( srv );
// Check sampling of random variable #1
computed = mean(sampling(1:np,1));
expected = 1.0;
assert_close ( computed , expected , 1.e-15 );
// Check sampling of random variable #2
computed = mean(sampling(1:np,2));
expected = 1.75;
assert_close ( computed , expected , 1.e-15 );
setrandvar_destroy(srv);
randvar_destroy(rv1);
randvar_destroy(rv2);

// Test setrandvar_buildsample with ( name , np ) and with QmcSobol
nisp_initseed ( 0 );
srv = setrandvar_new ( );
rv1 = randvar_new("Normale",1.0,0.5);
rv2 = randvar_new("Uniforme",1.0,2.5);
setrandvar_addrandvar ( srv , rv1 );
setrandvar_addrandvar ( srv , rv2 );
np = 1000;
name = "QmcSobol";
setrandvar_buildsample ( srv , name , np );
sampling = setrandvar_getsample ( srv );
// Check sampling of random variable #1
computed = mean(sampling(1:np,1));
expected = 1.0;
assert_close ( computed , expected , 1.e-3 );
// Check sampling of random variable #2
computed = mean(sampling(1:np,2));
expected = 1.75;
assert_close ( computed , expected , 1.e-3 );
setrandvar_destroy(srv);
randvar_destroy(rv1);
randvar_destroy(rv2);

// TODO : test with ( name , np ) and names :
//   name = "Quadrature"
//   name = "Petras"
//   name = "SmolyakGauss"
//   name = "SmolyakTrapeze"
//   name = "SmolyakFejer"
//   name = "SmolyakClenshawCurtis"

// Test setrandvar_buildsample with ( name , np , ne ) and with MonteCarlo
srv = setrandvar_new ( );
rv1 = randvar_new("Normale",1.0,0.5);
rv2 = randvar_new("Uniforme",1.0,2.5);
setrandvar_addrandvar ( srv , rv1 );
setrandvar_addrandvar ( srv , rv2 );
np = 1000;
ne = 10;
name = "MonteCarlo";
// setrandvar_buildsample ( srv , name , np , ne ); > Generates an error which cannot be tested
setrandvar_destroy(srv);
randvar_destroy(rv1);
randvar_destroy(rv2);

// Test setrandvar_buildsample with ( name , np , ne ) and with LhsMaxMin
srv = setrandvar_new ( );
rv1 = randvar_new("Normale",1.0,0.5);
rv2 = randvar_new("Uniforme",1.0,2.5);
setrandvar_addrandvar ( srv , rv1 );
setrandvar_addrandvar ( srv , rv2 );
np = 1000;
ne = 10;
name = "LhsMaxMin";
setrandvar_buildsample ( srv , name , np , ne ); 
sampling = setrandvar_getsample ( srv );
// Check sampling of random variable #1
computed = mean(sampling(1:np,1));
expected = 1.0;
assert_close ( computed , expected , 1.e-3 );
// Check sampling of random variable #2
computed = mean(sampling(1:np,2));
expected = 1.75;
assert_close ( computed , expected , 1.e-3 );
setrandvar_destroy(srv);
randvar_destroy(rv1);
randvar_destroy(rv2);

// Test setrandvar_buildsample with a source setrandvar
nisp_initseed ( 0 );
srv = setrandvar_new ( );
rv1 = randvar_new("Normale",1.0,0.5);
rv2 = randvar_new("Uniforme",1.0,2.5);
setrandvar_addrandvar ( srv , rv1 );
setrandvar_addrandvar ( srv , rv2 );
np = 1000;
name = "MonteCarlo";
setrandvar_buildsample ( srv , name , np ); 
srv2 = setrandvar_new ( );
setrandvar_addrandvar ( srv2 , rv1 );
setrandvar_addrandvar ( srv2 , rv2 );
setrandvar_buildsample ( srv2 , srv ); 
sampling = setrandvar_getsample ( srv2 );
// Check sampling of random variable #1
computed = mean(sampling(1:np,1));
expected = 1.0;
assert_close ( computed , expected , 1.e-1 );
// Check sampling of random variable #2
computed = mean(sampling(1:np,2));
expected = 1.75;
assert_close ( computed , expected , 1.e-1 );
setrandvar_destroy(srv2);
setrandvar_destroy(srv);
randvar_destroy(rv1);
randvar_destroy(rv2);


